import { randomInt, Repository } from '../utils/index.mjs';
import DbSeeds from '../seeds/data.json' assert { type: 'json' };
import { main } from '../../appSetup.mjs';

let initialized;
export const Server = () => {
    const serverStart = () => {
        return main();
    }
    
    return {
        init: () => {
            if (!initialized) {
                initialized = serverStart();
            }
            return initialized;
        }
    }
}

/**
 * @function
 * @param {String} mac - hardware device MAC address
 * @return {Object}
 */
export function createData(mac) {
    return {
        mac,
        ts: new Date().toISOString(),
        hex: '0001',
    };
}

/**
 * @function
 * @param {String} mac - hardware device MAC address
 * @param {Number} sublocationId - sublocation numeric identifier
 * @return {Object}
 */
export function createPosition(mac, sublocationId) {
    return {
        mac,
        ts: new Date().toISOString(),
        data: {
            x: randomInt(),
            y: randomInt(),
            z: randomInt(),
            sl_id: sublocationId,
            fom: randomInt(),
            slvr_mode: 'UNKNOWN',
            inactive: true
        },
    };
}

/**
 *
 * @param {String} mac
 * @param {String} name - event name
 * @param {Number} sublocationId - sublocation numeric identifier
 * @return {Object}
 */
export function createEvent(name, mac, sublocationId) {
    return {
        mac,
        ts: new Date().toISOString(),
        type: name,
        data: {
            hb_period: 30,
            pos_freq: 2,
            ntype: 'BRIDGE',
            inactive: true,
            charging: true,
            sync: true,
            batlow: false,
            sublocation_id: sublocationId,
            parent: 'E4956EA501fc',
            bootup: 5,
            node_id_req: true,
            external_reset: true,
            watchdog_reset: false,
            software_reset: false,
            x: randomInt(),
            y: randomInt(),
            z: randomInt(),
            sl_z: 1000,
        },
    };
}

export async function initRepoData() {
    const repository = new Repository();
    for (const trackable of DbSeeds.trackables) {
        await repository.createTrackable(trackable);
    }
    for (const association of DbSeeds.associations) {
        await repository.createAssociation(association);
    }
    for (const node of DbSeeds.nodes) {
        await repository.createNode(node);
    }
    for (const sublocation of DbSeeds.sublocations) {
        await repository.createSublocation(sublocation);
    }
}
export async function delay(ms) {
    return new Promise((resolve) => setTimeout(resolve, ms));
}

export async function findNodeByMac(mac) {
    const repository = new Repository();
    return repository.findNodeByMac(mac);
}
export async function findTrackableByUid(uid) {
    const repository = new Repository();
    return repository.findTrackableByUid(uid);
}

export async function deleteNodeByMac(node) {
    const repository = new Repository();
    return repository.deleteNode(node);
}

export async function clearRepoData() {
    const repository = new Repository();

    for (const trackable of DbSeeds.trackables) {
        await repository.deleteTrackable(trackable.uid);
    }
    for (const association of DbSeeds.associations) {
        await repository.deleteAssociation(association);
    }
    for (const node of DbSeeds.nodes) {
        await repository.deleteNode(node);
    }
    for (const sublocation of DbSeeds.sublocations) {
        await repository.deleteSublocation(sublocation.uid);
    }
}
