import { createRepository } from '@rtls-platform/repository/index.mjs';
import { createDbConnection } from '../../lib/db/index.mjs';
import * as ConfigContainer from '../../lib/config.cjs';

export class Repository {
    /**
     * @typedef {Class} Repository - repository interface for creation test prerequisites
     */

    /**
     * @property {Object|null}
     */
    #repo = null;
    /**
     * @property {Object|null}
     */
    #trackable = null;
    /**
     * @property {Object|null}
     */
    #anchor = null;
    /**
     * @property {Object|null}
     */
    #sublocation = null;
    /**
     * @property {Object|null}
     */
    #node = null;
    /**
     * @property {Object|null}
     */
    #association = null;

    constructor() {
        this.#repo = createRepository({
            repositoryInstance: createDbConnection({
                port: ConfigContainer.config.db.port,
                host: ConfigContainer.config.db.host,
            }),
            prefix: ConfigContainer.config.db.prefix,
        });
        this.#trackable = this.#repo.trackable;
        this.#anchor = this.#repo.anchor;
        this.#sublocation = this.#repo.sublocation;
        this.#node = this.#repo.node;
        this.#association = this.#repo.association;
    }

    /**
     * @method
     * @return {VoidFunction}
     */
    async createTrackable(data) {
        await this.#trackable.save(data);
    }

    /**
     * @method
     * @return {VoidFunction}
     */
    async deleteTrackable(uid) {
        await this.#trackable.deleteByUid(uid);
    }

    /**
     * @method
     * @return {VoidFunction}
     */
    async findTrackableByUid(uid) {
        return this.#trackable.findByUid(uid);
    }

    /**
     * @method
     * @return {VoidFunction}
     */
    async createNode(data) {
        await this.#node.save(data);
    }

    /**
     * @method
     * @param {String} mac
     * @return {VoidFunction}
     */
    async findNodeByMac(mac) {
        return await this.#node.findByMac(mac);
    }
    /**
     * @method
     * @return {VoidFunction}
     */
    async deleteNode(node) {
        await this.#node.del(node);
    }

    /**
     * @method
     * @return {VoidFunction}
     */
    async createAssociation(data) {
        await this.#association.save(data);
    }

    /**
     * @method
     * @return {VoidFunction}
     */
    async deleteAssociation(data) {
        await this.#association.del(data);
    }

    /**
     * @method
     * @return {VoidFunction}
     */
    async createSublocation(data) {
        await this.#sublocation.save(data);
    }

    /**
     * @method
     * @return {VoidFunction}
     */
    async deleteSublocation(uid) {
        await this.#sublocation.deleteByUid(uid);
    }
}
