import express from 'express';
import * as swagger from './swagger.mjs';
import middlewares from './middlewares.mjs';
import v3Router from './v3/router.mjs';
import logger from '../logger.mjs';

let server;
export const app = express();

app.use(middlewares.cors);
app.use(middlewares.json);
app.use(middlewares.urlencoded);
app.use(middlewares.cookieParser);
app.use(middlewares.static);

app.use('/api/v3/', v3Router);
app.use('/api-docs', swagger.serve, swagger.setup);
app.get('/', (req, res) => {
    res.status(200).send({});
});

export function startServer({ serverPort }) {
    server = app.listen(serverPort, () => {
        logger.info('Server listening on port:', serverPort);
    });
}

export async function stopServer() {
    if (!server) return;

    server.close();
    logger.info('Server stopped');
}
