/**
 * @function
 * @param {Object} trackable - database trackable
 * @return {Object} - trackable
 */
export function formatTrackableResponse(trackable) {
    const responseTrackable = { ...trackable };

    if (!('tag_mac' in responseTrackable)) {
        responseTrackable.tag_mac = null;
    }

    if (responseTrackable.position_reporting) {
        const { interval, auto } = responseTrackable.position_reporting;
        responseTrackable.position_reporting.interval = interval ? Number(interval) : 0;
        responseTrackable.position_reporting.auto = auto ? !!Number(auto) : false;
    } else {
        responseTrackable.position_reporting = { interval: 0, auto: true };
    }

    if (!('tag_mount_height' in responseTrackable)) {
        responseTrackable.tag_mount_height = null;
    } else {
        responseTrackable.tag_mount_height = Number(responseTrackable.tag_mount_height);
    }

    if (responseTrackable.heartbeat) {
        responseTrackable.heartbeat = responseTrackable.heartbeat.toUpperCase();
    }
    
    if (!responseTrackable.position) {
        responseTrackable.position = null;
    } else {
        responseTrackable.position.x = Number(responseTrackable.position.x);
        responseTrackable.position.y = Number(responseTrackable.position.y);
        responseTrackable.position.z = Number(responseTrackable.position.z);
        if ('fom' in responseTrackable.position) {
            responseTrackable.position.fom = Number(responseTrackable.position.fom);
        }
    }

    return {
        ...responseTrackable,
    };
}

/**
 * @function
 * @param {Array<Object>} trackables - database trackables
 * @return {Array<Object>} - trackables
 */
export function formatTrackableResponseList(trackables) {
    return trackables.map(formatTrackableResponse);
}
