import UseCaseBase from '../UseCaseBase.mjs';
import { Trackable } from '../../models/index.mjs';

export class TrackableSave extends UseCaseBase {
    static validationRules = {
        name: ['strict_string', 'not_empty', 'required'],
        tag_mount_height: ['strict_integer', { default: 0 }],
        heartbeat: ['not_empty', { one_of: ['ENABLED', 'DISABLED', 'enabled', 'disabled'] }, { default: 'ENABLED' }, 'to_upper_case'],
        position_reporting: [{
            nested_object: {
                interval: ['strict_decimal', { multiple: 0.25 }, { min_number: 0 }, 'required'],
                auto: ['strict_bool', 'required']
            }
        }, 'required' ],
        position: [{
            nested_object: {
                x: ['strict_integer', 'required'],
                y: ['strict_integer', 'required'],
                z: ['strict_integer', 'required'],
                sublocation_uid: ['strict_string', 'not_empty', 'required'],
            }
        }]
    };

    async execute(params) {
        const trackable = await new Trackable().save(params);

        await this.notificator.notify(Trackable.savedMessage(trackable));

        return trackable;
    }
}
