import UseCaseBase from '../UseCaseBase.mjs';
import { Trackable } from '../../models/index.mjs';

export class TrackableUpdate extends UseCaseBase {
    static validationRules = {
        uid: ['strict_string', 'required'],
        name: ['strict_string', 'not_empty'],
        tag_mount_height: ['strict_integer'],
        heartbeat: ['not_empty', { one_of: ['enabled', 'disabled', 'ENABLED', 'DISABLED'] }, 'to_upper_case'],
        position_reporting: [{
            nested_object: {
                interval: ['strict_decimal', { multiple: 0.25 }, { min_number: 0 }, 'required'],
                auto: ['strict_bool', 'required']
            }
        }],
        position: [{
            nested_object: {
                x: ['strict_integer', 'required'],
                y: ['strict_integer', 'required'],
                z: ['strict_integer', 'required'],
                sublocation_uid: ['strict_string', 'not_empty', 'required'],
            }
        }]
    };

    async execute(params) {
        const trackable = await new Trackable().update(params);

        await this.notificator.notify(Trackable.updatedMessage(trackable));

        return trackable;
    }
}
