import ModelBase from '../ModelBase.mjs';
import { toISOTime } from '../../utils/index.mjs';

export class Anchor extends ModelBase {
    /**
     * @param anchor {SolutionAnchor}
     * @returns {PlatformAnchor}
     */
    static formatSolutionAnchor(anchor) {
        return {
            uid: anchor.uid,
            name: anchor.name,
            bridge: (anchor.win_type_id === 4) ? 1 : 0,
            created_at: toISOTime(anchor.created_at),
            updated_at: toISOTime(anchor.updated_at),
            position: {
                x: anchor.position_x,
                y: anchor.position_y,
                z: anchor.position_z,
                sublocation_uid: anchor.sublocation_uid,
            },
        }
    }

    /**
     *
     * @param anchor {PlatformAnchor}
     * @returns {PlatformAnchor}
     */
    static formatPlatformAnchor(anchor) {
        return {
            uid: anchor.uid,
            name: anchor.name,
            bridge: Number(anchor.bridge),
            created_at: anchor.created_at,
            updated_at: anchor.updated_at,
            position: anchor.position,
        }
    }

    /**
     * @return {Promise<PlatformAnchor[]>}
     */
    async collectSolutionEntities() {
        return (await this.solutionRepositoryInstance.anchor.readAll()).map(Anchor.formatSolutionAnchor)
    }

    /**
     *
     * @returns {Promise<PlatformAnchor[]>}
     */
    async collectPlatformEntities() {
        return (await this.platformRepositoryInstance.anchor.findAll()).map(Anchor.formatPlatformAnchor);
    }

    /**
     *
     * @param items {PlatformAnchor[]}
     * @returns {Promise<void>}
     */
    async addToPlatform(items) {
        for (const item of items) {
            await this.platformRepositoryInstance.anchor.save(item);
        }
    }

    /**
     *
     * @param items {PlatformAnchor[]}
     * @returns {Promise<void>}
     */
    async removeInPlatform(items) {
        for (const item of items) {
            await this.platformRepositoryInstance.anchor.del(item);
        }
    }

    /**
     *
     * @param items {PlatformAnchor[]}
     * @returns {Promise<void>}
     */
    async updateInPlatform(items) {
        for (const item of items) {
            await this.platformRepositoryInstance.anchor.save(item);
        }
    }
}
