import ModelBase from '../ModelBase.mjs';
import { parseSafe, toISOTime } from '../../utils/index.mjs';

export class Trackable extends ModelBase {
    /**
     *
     * @param tko {SolutionTrackable}
     * @returns {PlatformTrackable}
     */
    static formatSolutionTko(tko) {
        let position_reporting = {
            auto: '1', // todo ?
            interval: 0 // todo ?
        };
        if (tko.stationary) {
            position_reporting.interval = 0;
        } else if (tko.pos_report_frequency > 0) {
            position_reporting.interval = 1 / (tko.pos_report_frequency / 30); // todo ?
        }
        const raw_position = !tko.position ? undefined : parseSafe(tko.position);
        const position = !raw_position ? undefined : {
            x: raw_position.x,
            y: raw_position.y,
            z: raw_position.z,
            sublocation_uid: raw_position.sl_id
        };

        return {
            name: tko.name,
            uid: tko.uid,
            tag_mount_height: tko.default_height || 0,
            heartbeat: tko.to_type_id === 6 ? 'DISABLED' : 'ENABLED', // todo ?
            position_reporting,
            position,
            created_at: toISOTime(tko.created_at),
            updated_at: toISOTime(tko.updated_at),
        };
    }
    
    /**
     *
     * @param tko {PlatformTrackable}
     * @returns {PlatformTrackable}
     */
    static formatPlatformTko(tko) {
        const interval = tko.position_reporting && tko.position_reporting.interval || 0;
        const auto = tko.position_reporting && tko.position_reporting.auto || true;
        return {
            name: tko.name,
            uid: tko.uid,
            tag_mount_height: isNaN(tko.tag_mount_height) ? 0 : Number(tko.tag_mount_height),
            heartbeat: tko.heartbeat,
            position_reporting: {
                auto,
                interval: Number(interval)
            },
            position: tko.position,
            created_at: tko.created_at,
            updated_at: tko.updated_at,
        }
    }
    
    /**
     *
     * @returns {Promise<SolutionTrackable[]>}
     */
    async collectSolutionEntities() {
        return (await this.solutionRepositoryInstance.trackable.readAll()).map(Trackable.formatSolutionTko);
    }
    
    /**
     *
     * @returns {Promise<PlatformTrackable[]>}
     */
    async collectPlatformEntities() {
        return (await this.platformRepositoryInstance.trackable.findAll()).map(Trackable.formatPlatformTko);
    }
    
    /**
     *
     * @param items {PlatformTrackable[]}
     * @returns {Promise<void>}
     */
    async addToPlatform(items) {
        for (const item of items) {
            await this.platformRepositoryInstance.trackable.save(item);
        }
    }
    
    /**
     *
     * @param items {PlatformTrackable[]}
     * @returns {Promise<void>}
     */
    async removeInPlatform(items) {
        for (const item of items) {
            await this.platformRepositoryInstance.trackable.del(item);
        }
    }
    
    /**
     *
     * @param items {PlatformTrackable[]}
     * @returns {Promise<void>}
     */
    async updateInPlatform(items) {
        for (const item of items) {
            await this.platformRepositoryInstance.trackable.save(item);
        }
    }
}
