import RepoBase from '../RepoBase.mjs';

/**
 * @typedef SolutionLocation {Object}
 * @property uid {String}
 * @property name {String}
 * @property created_at {Date}
 * @property updated_at {Date}
 */

/**
 * @class LocationRepository
 * @description create access for the locations in the solutions DB
 */
export class LocationRepository extends RepoBase {
    /**
     * @param {Object} options - repository options
     * @param {Object} options.repositoryInstance - database connection
     */
    constructor(options) {
        const { repositoryInstance } = options;
        super(repositoryInstance);
    }

    /**
     * @method
     * @return {Promise<SolutionLocation[]>} returns result of search by key
     */
    async readAll() {
        return (await this.repoInstance.queryAsync(LocationRepository.readAllSql) || []);
    }
    
    static get readAllSql() {
        return `select
                    uid, name, created_at, updated_at
                from view_locations;`;
    }
}
