import sinon from 'sinon';
import assert from 'assert';
import { SolutionDBPoolFactory } from '../lib/db/solutionDBClient.js';
import dbData from './solution-db-data.json' assert { type: 'json' };
import { fakeQueryAsync, Service } from './helpers.mjs';

describe('Anchors transfer', function() {
    let solutionPoolStub;
    let platformRepository;
    let resultAnchors;
    
    before(async function() {
        solutionPoolStub = sinon.stub(SolutionDBPoolFactory, 'createSolutionDBPool');
        solutionPoolStub.callsFake(() => ({ queryAsync: fakeQueryAsync }));
        const result = await Service().init();
        platformRepository = result.platformRepository;
    });
    after(async function() {
        SolutionDBPoolFactory.createSolutionDBPool.restore();
        if (Array.isArray(resultAnchors)) {
            for (const anchor of resultAnchors) {
                await platformRepository.anchor.del({ uid: anchor.uid });
            }
        }
    });

    it('Move anchors', async function() {
        const resultAnchors = await platformRepository.anchor.findAll();

        let anchor = resultAnchors.find((anchor) => anchor.name === 'Anchor: Lab 2');
        assert.deepStrictEqual(anchor, dbData.anchors.expect.anchor1);
        anchor = resultAnchors.find((anchor) => anchor.name === 'Bridge B5');
        assert.deepStrictEqual(anchor, dbData.anchors.expect.anchor2);
        anchor = resultAnchors.find((anchor) => anchor.name === 'Hallway ICM');
        assert.deepStrictEqual(anchor, dbData.anchors.expect.anchor3);
        anchor = resultAnchors.find((anchor) => anchor.name === 'Autocreated');
        assert.deepStrictEqual(anchor, dbData.anchors.expect.anchor4);
    });
});
