import sinon from 'sinon';
import assert from 'assert';
import { SolutionDBPoolFactory } from '../lib/db/solutionDBClient.js';
import dbData from './solution-db-data.json' assert { type: 'json' };
import { fakeQueryAsync, Service } from './helpers.mjs';

describe('Sublocations transfer', function() {
    let solutionPoolStub;
    let platformRepository;
    let resultSublocations;
    
    before(async function() {
        solutionPoolStub = sinon.stub(SolutionDBPoolFactory, 'createSolutionDBPool');
        solutionPoolStub.callsFake(() => ({ queryAsync: fakeQueryAsync }));
        const result = await Service().init();
        platformRepository = result.platformRepository;
    });
    after(async function() {
        if (Array.isArray(resultSublocations)) {
            for (const sublocation of resultSublocations) {
                await platformRepository.sublocation.deleteByUid(sublocation.uid);
            }
        }
        SolutionDBPoolFactory.createSolutionDBPool.restore();
    });
    
    it('Move sublocations', async function() {
        resultSublocations = await platformRepository.sublocation.findAll();

        let sublocation = resultSublocations.find((loc) => loc.name === 'OfficeSpace');
        dbData.sublocations.expect.sublocation3.id = sublocation.id;
        assert.deepStrictEqual(sublocation, dbData.sublocations.expect.sublocation3);

        sublocation = resultSublocations.find((loc) => loc.name === 'WebDev');
        dbData.sublocations.expect.sublocation2.id = sublocation.id;
        assert.deepStrictEqual(sublocation, dbData.sublocations.expect.sublocation2);

        sublocation = resultSublocations.find((loc) => loc.name === 'Hallway_host');
        dbData.sublocations.expect.sublocation1.id = sublocation.id;
        assert.deepStrictEqual(sublocation, dbData.sublocations.expect.sublocation1);
    });
});
